/*
 Copyright (c) 2009, Matt Wright All rights reserved.
 
 Redistribution and use in source and binary forms, with or without modification, are permitted provided that the
 following conditions are met:
 
 Redistributions of source code must retain the above copyright notice, this list of conditions and the following
 disclaimer. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 the following disclaimer in the documentation and/or other materials provided with the distribution. Neither the
 name of the Matt Wright nor the names of its contributors may be used to endorse or promote products derived from
 this software without specific prior written permission.
 
 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
 INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#import "ChooChooController.h"
#import <fcntl.h>
#import <WiganWallgate/dispatch.h>

@implementation ChooChooController

- (void)awakeFromNib
{  
  int fd = open("/Users/matt/Desktop/foo.txt", O_RDONLY);
  dispatch_source_t source = dispatch_source_create(DISPATCH_SOURCE_TYPE_READ, fd, 0, dispatch_get_global_queue(DISPATCH_QUEUE_PRIORITY_HIGH, 0));
  dispatch_source_set_event_handler(source, ^{
    char foo[1000];
    
    int num = read(fd, foo, 999);
    foo[num] = '\0';
    fprintf(stderr, foo);
  });
  //dispatch_resume(source);
  
  dispatch_source_t source2 = dispatch_source_create(DISPATCH_SOURCE_TYPE_TIMER, 0, 0, dispatch_get_global_queue(DISPATCH_QUEUE_PRIORITY_HIGH, 0));
  dispatch_source_set_timer(source2, dispatch_time(DISPATCH_TIME_NOW, 2*NSEC_PER_SEC), 2*NSEC_PER_SEC, 0);
  dispatch_source_set_event_handler(source2, ^{
    fprintf(stderr, "timer!\n");
    //dispatch_source_cancel(source2);
  });
  dispatch_resume(source2);
  
  dispatch_source_t source3 = dispatch_source_create(DISPATCH_SOURCE_TYPE_DATA_ADD, 0, 0, dispatch_get_global_queue(DISPATCH_QUEUE_PRIORITY_DEFAULT, 0));
  dispatch_source_set_event_handler(source3, ^{
    NSLog(@"merge data: %d", dispatch_source_get_data(source3));
  });
  dispatch_source_merge_data(source3, 100);
  
  dispatch_after(dispatch_time(DISPATCH_TIME_NOW, 2*NSEC_PER_SEC), dispatch_get_global_queue(DISPATCH_QUEUE_PRIORITY_DEFAULT, 0), ^{
    NSLog(@"done after");
  });
  
  dispatch_queue_t queue = dispatch_queue_create("foo", 0);
  
  dispatch_async(queue, ^{
    NSLog(@"serial 1");
  });
  
  dispatch_sync(dispatch_get_global_queue(0, 0), ^{
    NSLog(@"argh");
  });
  
  dispatch_async(dispatch_get_global_queue(0, 0), ^{
    NSLog(@"async arffhh");
  });
  dispatch_async(dispatch_get_global_queue(0, 0), ^{
    NSLog(@"async arffhh");
  });
  
  dispatch_async(dispatch_get_main_queue(), ^{
    NSLog(@"serial 2");
  });
  
  dispatch_async(queue, ^{
    NSLog(@"serial 3");
  });
  
  dispatch_group_t group = dispatch_group_create();
  dispatch_group_async(group, dispatch_get_global_queue(0, 0), ^{
    sleep(5);
    NSLog(@"Moo");
  });
  
  dispatch_group_notify(group, dispatch_get_global_queue(DISPATCH_QUEUE_PRIORITY_DEFAULT, 0), ^{
    NSLog(@"done");
  });
  
  //NSLog(@"waited, %d", dispatch_group_wait(group, 10000000000));
  
  dispatch_apply(10, dispatch_get_global_queue(DISPATCH_QUEUE_PRIORITY_DEFAULT, 0), ^(size_t i){
    printf("moo %d\n", i);
  });
  
  dispatch_release(queue);
  
  NSLog(@"quack, %llu, %llu", dispatch_time(DISPATCH_TIME_NOW, 0), dispatch_walltime(NULL, 0));
  
}

@end
