/*
 Wigan Wallgate: An open-source implementation of Grand Central Dispatch
 
 Copyright (c) 2009 Matt Wright
 
 Permission is hereby granted, free of charge, to any person obtaining a copy
 of this software and associated documentation files (the "Software"), to deal
 in the Software without restriction, including without limitation the rights
 to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 copies of the Software, and to permit persons to whom the Software is
 furnished to do so, subject to the following conditions:
 
 The above copyright notice and this permission notice shall be included in
 all copies or substantial portions of the Software.
 
 THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 THE SOFTWARE.
 */

#include <WiganWallgate/time.h>
#include <mach/mach_time.h>
#include <stdio.h>

dispatch_time_t dispatch_time(dispatch_time_t when, int64_t delta)
{
  if (when == DISPATCH_TIME_NOW)
  {
    when = mach_absolute_time();
  }
  when += delta;
  
  return when;
}

dispatch_time_t dispatch_walltime(const struct timespec *when, int64_t delta)
{
  mach_timebase_info_data_t info;
  mach_timebase_info(&info);

  struct timeval now;
  gettimeofday(&now, 0);
  
  // get the "now" in an absolute time kinda way
  uint64_t now_in_nsec = ((now.tv_sec * NSEC_PER_SEC) + (now.tv_usec * NSEC_PER_USEC) * (info.denom / info.numer));
  
  // get now in a mach time way
  uint64_t now_in_mach = mach_absolute_time();
  
  // work out the offset
  uint64_t offset = now_in_nsec - now_in_mach;
  
  // declare an absolute time in which to return, we'll return zero if the time offset underflows
  // we can't as the os to "wait" for less than current time anyway, can we?
  uint64_t result = 0;
  
  if (when)
  {
    uint64_t spec = ((when->tv_sec * NSEC_PER_SEC) + when->tv_nsec) * (info.denom / info.numer);
    
    // check to see if spec is within the range we can represent
    if (spec > (now_in_nsec - offset))
    {
      result = spec;
    }
  }
  else
  {
    result = now_in_nsec - offset;
  }
  
  result += delta;
    
  return result;
}